@@ -1,6 +1,9 @@
 from __future__ import annotations
 
-from datetime import timedelta
+from datetime import (
+    datetime,
+    timedelta,
+)
 import itertools
 
 import numpy as np
@@ -10,13 +13,10 @@ from pandas.compat import (
     IS64,
     is_platform_windows,
 )
+from pandas.compat.numpy import np_version_gt2
 
 import pandas as pd
 import pandas._testing as tm
-from pandas.core.api import (
-    Float64Index,
-    Int64Index,
-)
 
 ###############################################################
 # Index / Series common tests which may trigger dtype coercions
@@ -44,7 +44,6 @@ def check_comprehensiveness(request):
         yield
 
     else:
-
         for combo in combos:
             if not has_test(combo):
                 raise AssertionError(
@@ -55,7 +54,6 @@ def check_comprehensiveness(request):
 
 
 class CoercionBase:
-
     klasses = ["index", "series"]
     dtypes = [
         "object",
@@ -75,185 +73,11 @@ class CoercionBase:
 
 
 class TestSetitemCoercion(CoercionBase):
-
     method = "setitem"
 
-    def _assert_setitem_series_conversion(
-        self, original_series, loc_value, expected_series, expected_dtype
-    ):
-        """test series value's coercion triggered by assignment"""
-        temp = original_series.copy()
-        temp[1] = loc_value
-        tm.assert_series_equal(temp, expected_series)
-        # check dtype explicitly for sure
-        assert temp.dtype == expected_dtype
-
-        temp = original_series.copy()
-        temp.loc[1] = loc_value
-        tm.assert_series_equal(temp, expected_series)
-
-    @pytest.mark.parametrize(
-        "val,exp_dtype", [(1, object), (1.1, object), (1 + 1j, object), (True, object)]
-    )
-    def test_setitem_series_object(self, val, exp_dtype):
-        obj = pd.Series(list("abcd"))
-        assert obj.dtype == object
-
-        exp = pd.Series(["a", val, "c", "d"])
-        self._assert_setitem_series_conversion(obj, val, exp, exp_dtype)
-
-    @pytest.mark.parametrize(
-        "val,exp_dtype",
-        [(1, np.int64), (1.1, np.float64), (1 + 1j, np.complex128), (True, object)],
-    )
-    def test_setitem_series_int64(self, val, exp_dtype):
-        obj = pd.Series([1, 2, 3, 4])
-        assert obj.dtype == np.int64
-
-        exp = pd.Series([1, val, 3, 4])
-        self._assert_setitem_series_conversion(obj, val, exp, exp_dtype)
-
-    @pytest.mark.parametrize(
-        "val,exp_dtype", [(np.int32(1), np.int8), (np.int16(2**9), np.int16)]
-    )
-    def test_setitem_series_int8(self, val, exp_dtype):
-        obj = pd.Series([1, 2, 3, 4], dtype=np.int8)
-        assert obj.dtype == np.int8
-
-        warn = None if exp_dtype is np.int8 else FutureWarning
-        msg = "Values are too large to be losslessly cast to int8"
-        with tm.assert_produces_warning(warn, match=msg):
-            exp = pd.Series([1, val, 3, 4], dtype=np.int8)
-
-        exp = pd.Series([1, val, 3, 4], dtype=exp_dtype)
-        self._assert_setitem_series_conversion(obj, val, exp, exp_dtype)
-
-    @pytest.mark.parametrize(
-        "val,exp_dtype",
-        [(1, np.float64), (1.1, np.float64), (1 + 1j, np.complex128), (True, object)],
-    )
-    def test_setitem_series_float64(self, val, exp_dtype):
-        obj = pd.Series([1.1, 2.2, 3.3, 4.4])
-        assert obj.dtype == np.float64
-
-        exp = pd.Series([1.1, val, 3.3, 4.4])
-        self._assert_setitem_series_conversion(obj, val, exp, exp_dtype)
-
-    @pytest.mark.parametrize(
-        "val,exp_dtype",
-        [
-            (1, np.complex128),
-            (1.1, np.complex128),
-            (1 + 1j, np.complex128),
-            (True, object),
-        ],
-    )
-    def test_setitem_series_complex128(self, val, exp_dtype):
-        obj = pd.Series([1 + 1j, 2 + 2j, 3 + 3j, 4 + 4j])
-        assert obj.dtype == np.complex128
-
-        exp = pd.Series([1 + 1j, val, 3 + 3j, 4 + 4j])
-        self._assert_setitem_series_conversion(obj, val, exp, exp_dtype)
-
-    @pytest.mark.parametrize(
-        "val,exp_dtype",
-        [
-            (1, object),
-            ("3", object),
-            (3, object),
-            (1.1, object),
-            (1 + 1j, object),
-            (True, np.bool_),
-        ],
-    )
-    def test_setitem_series_bool(self, val, exp_dtype):
-        obj = pd.Series([True, False, True, False])
-        assert obj.dtype == np.bool_
-
-        exp = pd.Series([True, val, True, False], dtype=exp_dtype)
-        self._assert_setitem_series_conversion(obj, val, exp, exp_dtype)
-
-    @pytest.mark.parametrize(
-        "val,exp_dtype",
-        [(pd.Timestamp("2012-01-01"), "datetime64[ns]"), (1, object), ("x", object)],
-    )
-    def test_setitem_series_datetime64(self, val, exp_dtype):
-        obj = pd.Series(
-            [
-                pd.Timestamp("2011-01-01"),
-                pd.Timestamp("2011-01-02"),
-                pd.Timestamp("2011-01-03"),
-                pd.Timestamp("2011-01-04"),
-            ]
-        )
-        assert obj.dtype == "datetime64[ns]"
-
-        exp = pd.Series(
-            [
-                pd.Timestamp("2011-01-01"),
-                val,
-                pd.Timestamp("2011-01-03"),
-                pd.Timestamp("2011-01-04"),
-            ]
-        )
-        self._assert_setitem_series_conversion(obj, val, exp, exp_dtype)
-
-    @pytest.mark.parametrize(
-        "val,exp_dtype",
-        [
-            (pd.Timestamp("2012-01-01", tz="US/Eastern"), "datetime64[ns, US/Eastern]"),
-            (pd.Timestamp("2012-01-01", tz="US/Pacific"), object),
-            (pd.Timestamp("2012-01-01"), object),
-            (1, object),
-        ],
-    )
-    def test_setitem_series_datetime64tz(self, val, exp_dtype):
-        tz = "US/Eastern"
-        obj = pd.Series(
-            [
-                pd.Timestamp("2011-01-01", tz=tz),
-                pd.Timestamp("2011-01-02", tz=tz),
-                pd.Timestamp("2011-01-03", tz=tz),
-                pd.Timestamp("2011-01-04", tz=tz),
-            ]
-        )
-        assert obj.dtype == "datetime64[ns, US/Eastern]"
-
-        exp = pd.Series(
-            [
-                pd.Timestamp("2011-01-01", tz=tz),
-                val,
-                # once deprecation is enforced
-                # val if getattr(val, "tz", None) is None else val.tz_convert(tz),
-                pd.Timestamp("2011-01-03", tz=tz),
-                pd.Timestamp("2011-01-04", tz=tz),
-            ]
-        )
-        warn = None
-        if getattr(val, "tz", None) is not None and val.tz != obj[0].tz:
-            warn = FutureWarning
-        with tm.assert_produces_warning(warn, match="mismatched timezones"):
-            self._assert_setitem_series_conversion(obj, val, exp, exp_dtype)
-
-    @pytest.mark.parametrize(
-        "val,exp_dtype",
-        [(pd.Timedelta("12 day"), "timedelta64[ns]"), (1, object), ("x", object)],
-    )
-    def test_setitem_series_timedelta64(self, val, exp_dtype):
-        obj = pd.Series(
-            [
-                pd.Timedelta("1 day"),
-                pd.Timedelta("2 day"),
-                pd.Timedelta("3 day"),
-                pd.Timedelta("4 day"),
-            ]
-        )
-        assert obj.dtype == "timedelta64[ns]"
-
-        exp = pd.Series(
-            [pd.Timedelta("1 day"), val, pd.Timedelta("3 day"), pd.Timedelta("4 day")]
-        )
-        self._assert_setitem_series_conversion(obj, val, exp, exp_dtype)
+    # disable comprehensiveness tests, as most of these have been moved to
+    #  tests.series.indexing.test_setitem in SetitemCastingEquivalents subclasses.
+    klasses: list[str] = []
 
     def test_setitem_series_no_coercion_from_values_list(self):
         # GH35865 - int casted to str when internally calling np.array(ser.values)
@@ -269,12 +93,9 @@ class TestSetitemCoercion(CoercionBase):
     ):
         """test index's coercion triggered by assign key"""
         temp = original_series.copy()
-        warn = None
-        if isinstance(loc_key, int) and temp.index.dtype == np.float64:
-            # GH#33469
-            warn = FutureWarning
-        with tm.assert_produces_warning(warn):
-            temp[loc_key] = 5
+        # GH#33469 pre-2.0 with int loc_key and temp.index.dtype == np.float64
+        #  `temp[loc_key] = 5` treated loc_key as positional
+        temp[loc_key] = 5
         exp = pd.Series([1, 2, 3, 4, 5], index=expected_index)
         tm.assert_series_equal(temp, exp)
         # check dtype explicitly for sure
@@ -291,16 +112,18 @@ class TestSetitemCoercion(CoercionBase):
         "val,exp_dtype", [("x", object), (5, IndexError), (1.1, object)]
     )
     def test_setitem_index_object(self, val, exp_dtype):
-        obj = pd.Series([1, 2, 3, 4], index=list("abcd"))
+        obj = pd.Series([1, 2, 3, 4], index=pd.Index(list("abcd"), dtype=object))
         assert obj.index.dtype == object
 
         if exp_dtype is IndexError:
             temp = obj.copy()
+            warn_msg = "Series.__setitem__ treating keys as positions is deprecated"
             msg = "index 5 is out of bounds for axis 0 with size 4"
             with pytest.raises(exp_dtype, match=msg):
-                temp[5] = 5
+                with tm.assert_produces_warning(FutureWarning, match=warn_msg):
+                    temp[5] = 5
         else:
-            exp_index = pd.Index(list("abcd") + [val])
+            exp_index = pd.Index(list("abcd") + [val], dtype=object)
             self._assert_setitem_index_conversion(obj, val, exp_index, exp_dtype)
 
     @pytest.mark.parametrize(
@@ -314,23 +137,12 @@ class TestSetitemCoercion(CoercionBase):
         self._assert_setitem_index_conversion(obj, val, exp_index, exp_dtype)
 
     @pytest.mark.parametrize(
-        "val,exp_dtype", [(5, IndexError), (5.1, np.float64), ("x", object)]
+        "val,exp_dtype", [(5, np.float64), (5.1, np.float64), ("x", object)]
     )
     def test_setitem_index_float64(self, val, exp_dtype, request):
         obj = pd.Series([1, 2, 3, 4], index=[1.1, 2.1, 3.1, 4.1])
         assert obj.index.dtype == np.float64
 
-        if exp_dtype is IndexError:
-            # float + int -> int
-            temp = obj.copy()
-            msg = "index 5 is out of bounds for axis 0 with size 4"
-            with pytest.raises(exp_dtype, match=msg):
-                # GH#33469
-                depr_msg = "Treating integers as positional"
-                with tm.assert_produces_warning(FutureWarning, match=depr_msg):
-                    temp[5] = 5
-            mark = pytest.mark.xfail(reason="TODO_GH12747 The result must be float")
-            request.node.add_marker(mark)
         exp_index = pd.Index([1.1, 2.1, 3.1, 4.1, val])
         self._assert_setitem_index_conversion(obj, val, exp_index, exp_dtype)
 
@@ -364,7 +176,6 @@ class TestSetitemCoercion(CoercionBase):
 
 
 class TestInsertIndexCoercion(CoercionBase):
-
     klasses = ["index"]
     method = "insert"
 
@@ -385,42 +196,54 @@ class TestInsertIndexCoercion(CoercionBase):
         ],
     )
     def test_insert_index_object(self, insert, coerced_val, coerced_dtype):
-        obj = pd.Index(list("abcd"))
+        obj = pd.Index(list("abcd"), dtype=object)
         assert obj.dtype == object
 
-        exp = pd.Index(["a", coerced_val, "b", "c", "d"])
+        exp = pd.Index(["a", coerced_val, "b", "c", "d"], dtype=object)
         self._assert_insert_conversion(obj, insert, exp, coerced_dtype)
 
     @pytest.mark.parametrize(
         "insert, coerced_val, coerced_dtype",
         [
-            (1, 1, np.int64),
+            (1, 1, None),
             (1.1, 1.1, np.float64),
             (False, False, object),  # GH#36319
             ("x", "x", object),
         ],
     )
-    def test_insert_index_int64(self, insert, coerced_val, coerced_dtype):
-        obj = Int64Index([1, 2, 3, 4])
-        assert obj.dtype == np.int64
+    def test_insert_int_index(
+        self, any_int_numpy_dtype, insert, coerced_val, coerced_dtype
+    ):
+        dtype = any_int_numpy_dtype
+        obj = pd.Index([1, 2, 3, 4], dtype=dtype)
+        coerced_dtype = coerced_dtype if coerced_dtype is not None else dtype
 
-        exp = pd.Index([1, coerced_val, 2, 3, 4])
+        exp = pd.Index([1, coerced_val, 2, 3, 4], dtype=coerced_dtype)
         self._assert_insert_conversion(obj, insert, exp, coerced_dtype)
 
     @pytest.mark.parametrize(
         "insert, coerced_val, coerced_dtype",
         [
-            (1, 1.0, np.float64),
+            (1, 1.0, None),
+            # When float_numpy_dtype=float32, this is not the case
+            # see the correction below
             (1.1, 1.1, np.float64),
             (False, False, object),  # GH#36319
             ("x", "x", object),
         ],
     )
-    def test_insert_index_float64(self, insert, coerced_val, coerced_dtype):
-        obj = Float64Index([1.0, 2.0, 3.0, 4.0])
-        assert obj.dtype == np.float64
+    def test_insert_float_index(
+        self, float_numpy_dtype, insert, coerced_val, coerced_dtype
+    ):
+        dtype = float_numpy_dtype
+        obj = pd.Index([1.0, 2.0, 3.0, 4.0], dtype=dtype)
+        coerced_dtype = coerced_dtype if coerced_dtype is not None else dtype
 
-        exp = pd.Index([1.0, coerced_val, 2.0, 3.0, 4.0])
+        if np_version_gt2 and dtype == "float32" and coerced_val == 1.1:
+            # Hack, in the 2nd test case, since 1.1 can be losslessly cast to float32
+            # the expected dtype will be float32 if the original dtype was float32
+            coerced_dtype = np.float32
+        exp = pd.Index([1.0, coerced_val, 2.0, 3.0, 4.0], dtype=coerced_dtype)
         self._assert_insert_conversion(obj, insert, exp, coerced_dtype)
 
     @pytest.mark.parametrize(
@@ -435,21 +258,19 @@ class TestInsertIndexCoercion(CoercionBase):
         "insert_value",
         [pd.Timestamp("2012-01-01"), pd.Timestamp("2012-01-01", tz="Asia/Tokyo"), 1],
     )
-    def test_insert_index_datetimes(self, request, fill_val, exp_dtype, insert_value):
-
+    def test_insert_index_datetimes(self, fill_val, exp_dtype, insert_value):
         obj = pd.DatetimeIndex(
             ["2011-01-01", "2011-01-02", "2011-01-03", "2011-01-04"], tz=fill_val.tz
-        )
+        ).as_unit("ns")
         assert obj.dtype == exp_dtype
 
         exp = pd.DatetimeIndex(
             ["2011-01-01", fill_val.date(), "2011-01-02", "2011-01-03", "2011-01-04"],
             tz=fill_val.tz,
-        )
+        ).as_unit("ns")
         self._assert_insert_conversion(obj, fill_val, exp, exp_dtype)
 
         if fill_val.tz:
-
             # mismatched tzawareness
             ts = pd.Timestamp("2012-01-01")
             result = obj.insert(1, ts)
@@ -457,14 +278,11 @@ class TestInsertIndexCoercion(CoercionBase):
             assert expected.dtype == object
             tm.assert_index_equal(result, expected)
 
-            # mismatched tz --> cast to object (could reasonably cast to common tz)
             ts = pd.Timestamp("2012-01-01", tz="Asia/Tokyo")
-            with tm.assert_produces_warning(FutureWarning, match="mismatched timezone"):
-                result = obj.insert(1, ts)
+            result = obj.insert(1, ts)
             # once deprecation is enforced:
-            # expected = obj.insert(1, ts.tz_convert(obj.dtype.tz))
-            # assert expected.dtype == obj.dtype
-            expected = obj.astype(object).insert(1, ts)
+            expected = obj.insert(1, ts.tz_convert(obj.dtype.tz))
+            assert expected.dtype == obj.dtype
             tm.assert_index_equal(result, expected)
 
         else:
@@ -538,12 +356,6 @@ class TestInsertIndexCoercion(CoercionBase):
                 expected = obj.astype(object).insert(0, str(insert))
                 tm.assert_index_equal(result, expected)
 
-            msg = r"Unexpected keyword arguments {'freq'}"
-            with pytest.raises(TypeError, match=msg):
-                with tm.assert_produces_warning(FutureWarning):
-                    # passing keywords to pd.Index
-                    pd.Index(data, freq="M")
-
     @pytest.mark.xfail(reason="Test not implemented")
     def test_insert_index_complex128(self):
         raise NotImplementedError
@@ -554,8 +366,8 @@ class TestInsertIndexCoercion(CoercionBase):
 
 
 class TestWhereCoercion(CoercionBase):
-
     method = "where"
+    _cond = np.array([True, False, True, False])
 
     def _assert_where_conversion(
         self, original, cond, values, expected, expected_dtype
@@ -566,31 +378,35 @@ class TestWhereCoercion(CoercionBase):
         tm.assert_equal(res, expected)
         assert res.dtype == expected_dtype
 
+    def _construct_exp(self, obj, klass, fill_val, exp_dtype):
+        if fill_val is True:
+            values = klass([True, False, True, True])
+        elif isinstance(fill_val, (datetime, np.datetime64)):
+            values = pd.date_range(fill_val, periods=4)
+        else:
+            values = klass(x * fill_val for x in [5, 6, 7, 8])
+
+        exp = klass([obj[0], values[1], obj[2], values[3]], dtype=exp_dtype)
+        return values, exp
+
+    def _run_test(self, obj, fill_val, klass, exp_dtype):
+        cond = klass(self._cond)
+
+        exp = klass([obj[0], fill_val, obj[2], fill_val], dtype=exp_dtype)
+        self._assert_where_conversion(obj, cond, fill_val, exp, exp_dtype)
+
+        values, exp = self._construct_exp(obj, klass, fill_val, exp_dtype)
+        self._assert_where_conversion(obj, cond, values, exp, exp_dtype)
+
     @pytest.mark.parametrize(
         "fill_val,exp_dtype",
         [(1, object), (1.1, object), (1 + 1j, object), (True, object)],
     )
     def test_where_object(self, index_or_series, fill_val, exp_dtype):
         klass = index_or_series
-        obj = klass(list("abcd"))
+        obj = klass(list("abcd"), dtype=object)
         assert obj.dtype == object
-        cond = klass([True, False, True, False])
-
-        if fill_val is True and klass is pd.Series:
-            ret_val = 1
-        else:
-            ret_val = fill_val
-
-        exp = klass(["a", ret_val, "c", ret_val])
-        self._assert_where_conversion(obj, cond, fill_val, exp, exp_dtype)
-
-        if fill_val is True:
-            values = klass([True, False, True, True])
-        else:
-            values = klass(x * fill_val for x in [5, 6, 7, 8])
-
-        exp = klass(["a", values[1], "c", values[3]])
-        self._assert_where_conversion(obj, cond, values, exp, exp_dtype)
+        self._run_test(obj, fill_val, klass, exp_dtype)
 
     @pytest.mark.parametrize(
         "fill_val,exp_dtype",
@@ -598,23 +414,10 @@ class TestWhereCoercion(CoercionBase):
     )
     def test_where_int64(self, index_or_series, fill_val, exp_dtype, request):
         klass = index_or_series
-        if klass is pd.Index and exp_dtype is np.complex128:
-            mark = pytest.mark.xfail(reason="Complex Index not supported")
-            request.node.add_marker(mark)
 
         obj = klass([1, 2, 3, 4])
         assert obj.dtype == np.int64
-        cond = klass([True, False, True, False])
-
-        exp = klass([1, fill_val, 3, fill_val])
-        self._assert_where_conversion(obj, cond, fill_val, exp, exp_dtype)
-
-        if fill_val is True:
-            values = klass([True, False, True, True])
-        else:
-            values = klass(x * fill_val for x in [5, 6, 7, 8])
-        exp = klass([1, values[1], 3, values[3]])
-        self._assert_where_conversion(obj, cond, values, exp, exp_dtype)
+        self._run_test(obj, fill_val, klass, exp_dtype)
 
     @pytest.mark.parametrize(
         "fill_val, exp_dtype",
@@ -622,23 +425,10 @@ class TestWhereCoercion(CoercionBase):
     )
     def test_where_float64(self, index_or_series, fill_val, exp_dtype, request):
         klass = index_or_series
-        if klass is pd.Index and exp_dtype is np.complex128:
-            mark = pytest.mark.xfail(reason="Complex Index not supported")
-            request.node.add_marker(mark)
 
         obj = klass([1.1, 2.2, 3.3, 4.4])
         assert obj.dtype == np.float64
-        cond = klass([True, False, True, False])
-
-        exp = klass([1.1, fill_val, 3.3, fill_val])
-        self._assert_where_conversion(obj, cond, fill_val, exp, exp_dtype)
-
-        if fill_val is True:
-            values = klass([True, False, True, True])
-        else:
-            values = klass(x * fill_val for x in [5, 6, 7, 8])
-        exp = klass([1.1, values[1], 3.3, values[3]])
-        self._assert_where_conversion(obj, cond, values, exp, exp_dtype)
+        self._run_test(obj, fill_val, klass, exp_dtype)
 
     @pytest.mark.parametrize(
         "fill_val,exp_dtype",
@@ -649,42 +439,22 @@ class TestWhereCoercion(CoercionBase):
             (True, object),
         ],
     )
-    def test_where_series_complex128(self, fill_val, exp_dtype):
-        klass = pd.Series
-        obj = klass([1 + 1j, 2 + 2j, 3 + 3j, 4 + 4j])
+    def test_where_complex128(self, index_or_series, fill_val, exp_dtype):
+        klass = index_or_series
+        obj = klass([1 + 1j, 2 + 2j, 3 + 3j, 4 + 4j], dtype=np.complex128)
         assert obj.dtype == np.complex128
-        cond = klass([True, False, True, False])
-
-        exp = klass([1 + 1j, fill_val, 3 + 3j, fill_val])
-        self._assert_where_conversion(obj, cond, fill_val, exp, exp_dtype)
-
-        if fill_val is True:
-            values = klass([True, False, True, True])
-        else:
-            values = klass(x * fill_val for x in [5, 6, 7, 8])
-        exp = klass([1 + 1j, values[1], 3 + 3j, values[3]], dtype=exp_dtype)
-        self._assert_where_conversion(obj, cond, values, exp, exp_dtype)
+        self._run_test(obj, fill_val, klass, exp_dtype)
 
     @pytest.mark.parametrize(
         "fill_val,exp_dtype",
         [(1, object), (1.1, object), (1 + 1j, object), (True, np.bool_)],
     )
-    def test_where_series_bool(self, fill_val, exp_dtype):
-        klass = pd.Series
+    def test_where_series_bool(self, index_or_series, fill_val, exp_dtype):
+        klass = index_or_series
 
         obj = klass([True, False, True, False])
         assert obj.dtype == np.bool_
-        cond = klass([True, False, True, False])
-
-        exp = klass([True, fill_val, True, fill_val])
-        self._assert_where_conversion(obj, cond, fill_val, exp, exp_dtype)
-
-        if fill_val is True:
-            values = klass([True, False, True, True])
-        else:
-            values = klass(x * fill_val for x in [5, 6, 7, 8])
-        exp = klass([True, values[1], True, values[3]])
-        self._assert_where_conversion(obj, cond, values, exp, exp_dtype)
+        self._run_test(obj, fill_val, klass, exp_dtype)
 
     @pytest.mark.parametrize(
         "fill_val,exp_dtype",
@@ -694,112 +464,20 @@ class TestWhereCoercion(CoercionBase):
         ],
         ids=["datetime64", "datetime64tz"],
     )
-    def test_where_series_datetime64(self, fill_val, exp_dtype):
-        obj = pd.Series(
-            [
-                pd.Timestamp("2011-01-01"),
-                pd.Timestamp("2011-01-02"),
-                pd.Timestamp("2011-01-03"),
-                pd.Timestamp("2011-01-04"),
-            ]
-        )
+    def test_where_datetime64(self, index_or_series, fill_val, exp_dtype):
+        klass = index_or_series
+
+        obj = klass(pd.date_range("2011-01-01", periods=4, freq="D")._with_freq(None))
         assert obj.dtype == "datetime64[ns]"
-        cond = pd.Series([True, False, True, False])
 
-        exp = pd.Series(
-            [pd.Timestamp("2011-01-01"), fill_val, pd.Timestamp("2011-01-03"), fill_val]
-        )
-        self._assert_where_conversion(obj, cond, fill_val, exp, exp_dtype)
-
-        values = pd.Series(pd.date_range(fill_val, periods=4))
-        if fill_val.tz:
-            exp = pd.Series(
-                [
-                    pd.Timestamp("2011-01-01"),
-                    pd.Timestamp("2012-01-02 00:00", tz="US/Eastern"),
-                    pd.Timestamp("2011-01-03"),
-                    pd.Timestamp("2012-01-04 00:00", tz="US/Eastern"),
-                ]
-            )
-            self._assert_where_conversion(obj, cond, values, exp, exp_dtype)
-
-        exp = pd.Series(
-            [
-                pd.Timestamp("2011-01-01"),
-                values[1],
-                pd.Timestamp("2011-01-03"),
-                values[3],
-            ]
-        )
-        self._assert_where_conversion(obj, cond, values, exp, exp_dtype)
-
-    @pytest.mark.parametrize(
-        "fill_val",
-        [
-            pd.Timestamp("2012-01-01"),
-            pd.Timestamp("2012-01-01").to_datetime64(),
-            pd.Timestamp("2012-01-01").to_pydatetime(),
-        ],
-    )
-    def test_where_index_datetime(self, fill_val):
-        exp_dtype = "datetime64[ns]"
-        obj = pd.Index(
-            [
-                pd.Timestamp("2011-01-01"),
-                pd.Timestamp("2011-01-02"),
-                pd.Timestamp("2011-01-03"),
-                pd.Timestamp("2011-01-04"),
-            ]
-        )
-        assert obj.dtype == "datetime64[ns]"
-        cond = pd.Index([True, False, True, False])
-
-        result = obj.where(cond, fill_val)
-        expected = pd.DatetimeIndex([obj[0], fill_val, obj[2], fill_val])
-        tm.assert_index_equal(result, expected)
-
-        values = pd.Index(pd.date_range(fill_val, periods=4))
-        exp = pd.Index(
-            [
-                pd.Timestamp("2011-01-01"),
-                pd.Timestamp("2012-01-02"),
-                pd.Timestamp("2011-01-03"),
-                pd.Timestamp("2012-01-04"),
-            ]
-        )
-
-        self._assert_where_conversion(obj, cond, values, exp, exp_dtype)
-
-    def test_where_index_datetime64tz(self):
-        fill_val = pd.Timestamp("2012-01-01", tz="US/Eastern")
-        exp_dtype = object
-        obj = pd.Index(
-            [
-                pd.Timestamp("2011-01-01"),
-                pd.Timestamp("2011-01-02"),
-                pd.Timestamp("2011-01-03"),
-                pd.Timestamp("2011-01-04"),
-            ]
-        )
-        assert obj.dtype == "datetime64[ns]"
-        cond = pd.Index([True, False, True, False])
-
-        res = obj.where(cond, fill_val)
-        expected = pd.Index([obj[0], fill_val, obj[2], fill_val], dtype=object)
-        tm.assert_index_equal(res, expected)
-
-        values = pd.Index(pd.date_range(fill_val, periods=4))
-        exp = pd.Index(
-            [
-                pd.Timestamp("2011-01-01"),
-                pd.Timestamp("2012-01-02", tz="US/Eastern"),
-                pd.Timestamp("2011-01-03"),
-                pd.Timestamp("2012-01-04", tz="US/Eastern"),
-            ],
-            dtype=exp_dtype,
-        )
-
-        self._assert_where_conversion(obj, cond, values, exp, exp_dtype)
+        fv = fill_val
+        # do the check with each of the available datetime scalars
+        if exp_dtype == "datetime64[ns]":
+            for scalar in [fv, fv.to_pydatetime(), fv.to_datetime64()]:
+                self._run_test(obj, scalar, klass, exp_dtype)
+        else:
+            for scalar in [fv, fv.to_pydatetime()]:
+                self._run_test(obj, fill_val, klass, exp_dtype)
 
     @pytest.mark.xfail(reason="Test not implemented")
     def test_where_index_complex128(self):
@@ -842,7 +520,7 @@ class TestWhereCoercion(CoercionBase):
 
         cond = np.array([False, True, False])
 
-        # Passinga  valid scalar
+        # Passing a valid scalar
         value = pi[-1] + pi.freq * 10
         expected = pd.PeriodIndex([value, pi[1], value])
         result = pi.where(cond, value)
@@ -867,7 +545,6 @@ class TestWhereCoercion(CoercionBase):
 
 
 class TestFillnaSeriesCoercion(CoercionBase):
-
     # not indexing, but place here for consistency
 
     method = "fillna"
@@ -889,10 +566,10 @@ class TestFillnaSeriesCoercion(CoercionBase):
     )
     def test_fillna_object(self, index_or_series, fill_val, fill_dtype):
         klass = index_or_series
-        obj = klass(["a", np.nan, "c", "d"])
+        obj = klass(["a", np.nan, "c", "d"], dtype=object)
         assert obj.dtype == object
 
-        exp = klass(["a", fill_val, "c", "d"])
+        exp = klass(["a", fill_val, "c", "d"], dtype=object)
         self._assert_fillna_conversion(obj, fill_val, exp, fill_dtype)
 
     @pytest.mark.parametrize(
@@ -905,11 +582,6 @@ class TestFillnaSeriesCoercion(CoercionBase):
         assert obj.dtype == np.float64
 
         exp = klass([1.1, fill_val, 3.3, 4.4])
-        # float + complex -> we don't support a complex Index
-        # complex for Series,
-        # object for Index
-        if fill_dtype == np.complex128 and klass == pd.Index:
-            fill_dtype = object
         self._assert_fillna_conversion(obj, fill_val, exp, fill_dtype)
 
     @pytest.mark.parametrize(
@@ -921,11 +593,12 @@ class TestFillnaSeriesCoercion(CoercionBase):
             (True, object),
         ],
     )
-    def test_fillna_series_complex128(self, fill_val, fill_dtype):
-        obj = pd.Series([1 + 1j, np.nan, 3 + 3j, 4 + 4j])
+    def test_fillna_complex128(self, index_or_series, fill_val, fill_dtype):
+        klass = index_or_series
+        obj = klass([1 + 1j, np.nan, 3 + 3j, 4 + 4j], dtype=np.complex128)
         assert obj.dtype == np.complex128
 
-        exp = pd.Series([1 + 1j, fill_val, 3 + 3j, 4 + 4j])
+        exp = klass([1 + 1j, fill_val, 3 + 3j, 4 + 4j])
         self._assert_fillna_conversion(obj, fill_val, exp, fill_dtype)
 
     @pytest.mark.parametrize(
@@ -965,7 +638,8 @@ class TestFillnaSeriesCoercion(CoercionBase):
         [
             (pd.Timestamp("2012-01-01", tz="US/Eastern"), "datetime64[ns, US/Eastern]"),
             (pd.Timestamp("2012-01-01"), object),
-            (pd.Timestamp("2012-01-01", tz="Asia/Tokyo"), object),
+            # pre-2.0 with a mismatched tz we would get object result
+            (pd.Timestamp("2012-01-01", tz="Asia/Tokyo"), "datetime64[ns, US/Eastern]"),
             (1, object),
             ("x", object),
         ],
@@ -984,22 +658,43 @@ class TestFillnaSeriesCoercion(CoercionBase):
         )
         assert obj.dtype == "datetime64[ns, US/Eastern]"
 
+        if getattr(fill_val, "tz", None) is None:
+            fv = fill_val
+        else:
+            fv = fill_val.tz_convert(tz)
         exp = klass(
             [
                 pd.Timestamp("2011-01-01", tz=tz),
-                fill_val,
-                # Once deprecation is enforced, this becomes:
-                # fill_val.tz_convert(tz) if getattr(fill_val, "tz", None)
-                #  is not None else fill_val,
+                fv,
                 pd.Timestamp("2011-01-03", tz=tz),
                 pd.Timestamp("2011-01-04", tz=tz),
             ]
         )
-        warn = None
-        if getattr(fill_val, "tz", None) is not None and fill_val.tz != obj[0].tz:
-            warn = FutureWarning
-        with tm.assert_produces_warning(warn, match="mismatched timezone"):
-            self._assert_fillna_conversion(obj, fill_val, exp, fill_dtype)
+        self._assert_fillna_conversion(obj, fill_val, exp, fill_dtype)
+
+    @pytest.mark.parametrize(
+        "fill_val",
+        [
+            1,
+            1.1,
+            1 + 1j,
+            True,
+            pd.Interval(1, 2, closed="left"),
+            pd.Timestamp("2012-01-01", tz="US/Eastern"),
+            pd.Timestamp("2012-01-01"),
+            pd.Timedelta(days=1),
+            pd.Period("2016-01-01", "D"),
+        ],
+    )
+    def test_fillna_interval(self, index_or_series, fill_val):
+        ii = pd.interval_range(1.0, 5.0, closed="right").insert(1, np.nan)
+        assert isinstance(ii.dtype, pd.IntervalDtype)
+        obj = index_or_series(ii)
+
+        exp = index_or_series([ii[0], fill_val, ii[2], ii[3], ii[4]], dtype=object)
+
+        fill_dtype = object
+        self._assert_fillna_conversion(obj, fill_val, exp, fill_dtype)
 
     @pytest.mark.xfail(reason="Test not implemented")
     def test_fillna_series_int64(self):
@@ -1021,9 +716,29 @@ class TestFillnaSeriesCoercion(CoercionBase):
     def test_fillna_series_timedelta64(self):
         raise NotImplementedError
 
-    @pytest.mark.xfail(reason="Test not implemented")
-    def test_fillna_series_period(self):
-        raise NotImplementedError
+    @pytest.mark.parametrize(
+        "fill_val",
+        [
+            1,
+            1.1,
+            1 + 1j,
+            True,
+            pd.Interval(1, 2, closed="left"),
+            pd.Timestamp("2012-01-01", tz="US/Eastern"),
+            pd.Timestamp("2012-01-01"),
+            pd.Timedelta(days=1),
+            pd.Period("2016-01-01", "W"),
+        ],
+    )
+    def test_fillna_series_period(self, index_or_series, fill_val):
+        pi = pd.period_range("2016-01-01", periods=4, freq="D").insert(1, pd.NaT)
+        assert isinstance(pi.dtype, pd.PeriodDtype)
+        obj = index_or_series(pi)
+
+        exp = index_or_series([pi[0], fill_val, pi[2], pi[3], pi[4]], dtype=object)
+
+        fill_dtype = object
+        self._assert_fillna_conversion(obj, fill_val, exp, fill_dtype)
 
     @pytest.mark.xfail(reason="Test not implemented")
     def test_fillna_index_timedelta64(self):
@@ -1035,7 +750,6 @@ class TestFillnaSeriesCoercion(CoercionBase):
 
 
 class TestReplaceSeriesCoercion(CoercionBase):
-
     klasses = ["series"]
     method = "replace"
 
@@ -1117,9 +831,10 @@ class TestReplaceSeriesCoercion(CoercionBase):
             raise ValueError
         return replacer
 
-    def test_replace_series(self, how, to_key, from_key, replacer):
+    def test_replace_series(self, how, to_key, from_key, replacer, using_infer_string):
         index = pd.Index([3, 4], name="xxx")
         obj = pd.Series(self.rep[from_key], index=index, name="yyy")
+        obj = obj.astype(from_key)
         assert obj.dtype == from_key
 
         if from_key.startswith("datetime") and to_key.startswith("datetime"):
@@ -1129,12 +844,9 @@ class TestReplaceSeriesCoercion(CoercionBase):
             # tested below
             return
 
-        result = obj.replace(replacer)
-
         if (from_key == "float64" and to_key in ("int64")) or (
             from_key == "complex128" and to_key in ("int64", "float64")
         ):
-
             if not IS64 or is_platform_windows():
                 pytest.skip(f"32-bit platform buggy: {from_key} -> {to_key}")
 
@@ -1143,7 +855,21 @@ class TestReplaceSeriesCoercion(CoercionBase):
 
         else:
             exp = pd.Series(self.rep[to_key], index=index, name="yyy")
-            assert exp.dtype == to_key
+
+        if using_infer_string and exp.dtype == "string":
+            # with infer_string, we disable the deprecated downcasting behavior
+            exp = exp.astype(object)
+
+        msg = "Downcasting behavior in `replace`"
+        warn = FutureWarning
+        if (
+            exp.dtype == obj.dtype
+            or exp.dtype == object
+            or (exp.dtype.kind in "iufc" and obj.dtype.kind in "iufc")
+        ):
+            warn = None
+        with tm.assert_produces_warning(warn, match=msg):
+            result = obj.replace(replacer)
 
         tm.assert_series_equal(result, exp)
 
@@ -1155,15 +881,24 @@ class TestReplaceSeriesCoercion(CoercionBase):
     @pytest.mark.parametrize(
         "from_key", ["datetime64[ns, UTC]", "datetime64[ns, US/Eastern]"], indirect=True
     )
-    def test_replace_series_datetime_tz(self, how, to_key, from_key, replacer):
+    def test_replace_series_datetime_tz(
+        self, how, to_key, from_key, replacer, using_infer_string
+    ):
         index = pd.Index([3, 4], name="xyz")
         obj = pd.Series(self.rep[from_key], index=index, name="yyy")
         assert obj.dtype == from_key
 
-        result = obj.replace(replacer)
-
         exp = pd.Series(self.rep[to_key], index=index, name="yyy")
-        assert exp.dtype == to_key
+        if using_infer_string and exp.dtype == "string":
+            # with infer_string, we disable the deprecated downcasting behavior
+            exp = exp.astype(object)
+        else:
+            assert exp.dtype == to_key
+
+        msg = "Downcasting behavior in `replace`"
+        warn = FutureWarning if exp.dtype != object else None
+        with tm.assert_produces_warning(warn, match=msg):
+            result = obj.replace(replacer)
 
         tm.assert_series_equal(result, exp)
 
@@ -1182,24 +917,23 @@ class TestReplaceSeriesCoercion(CoercionBase):
         obj = pd.Series(self.rep[from_key], index=index, name="yyy")
         assert obj.dtype == from_key
 
-        warn = None
-        rep_ser = pd.Series(replacer)
-        if (
-            isinstance(obj.dtype, pd.DatetimeTZDtype)
-            and isinstance(rep_ser.dtype, pd.DatetimeTZDtype)
-            and obj.dtype != rep_ser.dtype
+        exp = pd.Series(self.rep[to_key], index=index, name="yyy")
+        warn = FutureWarning
+        if isinstance(obj.dtype, pd.DatetimeTZDtype) and isinstance(
+            exp.dtype, pd.DatetimeTZDtype
         ):
-            # mismatched tz DatetimeArray behavior will change to cast
-            #  for setitem-like methods with mismatched tzs GH#44940
-            warn = FutureWarning
+            # with mismatched tzs, we retain the original dtype as of 2.0
+            exp = exp.astype(obj.dtype)
+            warn = None
+        else:
+            assert exp.dtype == to_key
+            if to_key == from_key:
+                warn = None
 
-        msg = "explicitly cast to object"
+        msg = "Downcasting behavior in `replace`"
         with tm.assert_produces_warning(warn, match=msg):
             result = obj.replace(replacer)
 
-        exp = pd.Series(self.rep[to_key], index=index, name="yyy")
-        assert exp.dtype == to_key
-
         tm.assert_series_equal(result, exp)
 
     @pytest.mark.xfail(reason="Test not implemented")
